<xsl:stylesheet version="2.0"
				xmlns:ps="http://www.pageseeder.com/editing/2.0"
				xmlns:xhtml="http://www.w3.org/1999/xhtml"
				xmlns="http://www.w3.org/1999/xhtml"
				xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
				xmlns:dfx="http://www.topologi.com/2005/Diff-X"
                xmlns:psf="http://www.pageseeder.com/function"
				exclude-result-prefixes="ps xhtml dfx psf">

<xsl:import href="../../Layout/Common/page-functions.xsl" />
<!-- TODO: Check if the following import can be removed -->
<xsl:import href="../../Common/XRefUtilities.xsl" />

<xsl:param name="ps-release" select="'ps-default-value'" />
<xsl:param name="ps-releasestatus" select="'ps-default-value'" />
<xsl:param name="xsl-action" />
<xsl:param name="hosturl" />
<xsl:param name="siteprefix" />
<xsl:param name="stylegroupname" />
<xsl:param name="uriString"/>
<xsl:param name="filename"/>

<!-- 
	Definition lists should not have any spaces, otherwise indentation
	fails.
 -->
<xsl:strip-space elements="mlist"/>
 
<!-- required for spacing when the body is blank -->
<xsl:template match="body[not(*)]">
	<br/>
</xsl:template>

<xsl:template match="body">
  <xsl:variable name="type" select="concat('section-body',if (@type) then concat(' body-type-',@type) else '')"/>
  <div class="{$type}">
 	  <xsl:choose>
		<xsl:when test="@pageBreak='before' and not($xsl-action='preview')">
	    	<hr class="page-break" />
			<xsl:apply-templates/>
		</xsl:when>
		<xsl:when test="@pageBreak='after' and not($xsl-action='preview')">
			<xsl:apply-templates/>
			<hr class="page-break" />
		</xsl:when>
		<xsl:otherwise>
			<xsl:apply-templates/>
		</xsl:otherwise>
	  </xsl:choose>
	</div>
</xsl:template>

<xsl:template match="*[ancestor::customBody]">
    <xsl:copy-of select="."/>
</xsl:template>

 <!-- copy elements in customBody for history -->
<xsl:template match="*[ancestor::ps:diff]" priority="-1">
   <xsl:copy>
     <xsl:copy-of select="@*" />
     <xsl:apply-templates select="node()" />
   </xsl:copy>
</xsl:template>

<xsl:template match="link">
	<a>
		<xsl:if test="@href and not(starts-with(@href,'#'))">
			<xsl:attribute name="target">_parent</xsl:attribute>
		</xsl:if>
		<xsl:copy-of select="@name | @href" />
		<xsl:apply-templates/>
	</a>
  <!-- the non-breaking space is so that when editing in firefox you can add text after the link easily -->
	<xsl:if test="not(starts-with(normalize-space(following::text()[1]),'&#160;'))">&#160;</xsl:if>
</xsl:template>

<xsl:template match="blockXref">
  <xsl:call-template name="xref-content" />
</xsl:template>

<xsl:template match="xref">
  <xsl:call-template name="xref-content" />
</xsl:template>

<xsl:template name="xref-content">
  <xsl:param name="element-name" select="'a'" />
  <xsl:element name="{$element-name}">
    <xsl:attribute name="class">xref</xsl:attribute>
    <xsl:attribute name="manualtitle" select="@title" />
		<xsl:if test="@href and not(starts-with(@href,'#'))">
			<xsl:attribute name="target">_parent</xsl:attribute>
		</xsl:if>
		<xsl:copy-of select="@*[not(local-name()='title' or local-name() = 'href')]" />
		<xsl:attribute name="title">
      <xsl:value-of select="concat('Type: ', @type)"/>
      <xsl:if test="@labels">
        <xsl:value-of select="concat(' - Labels: ', string-join(tokenize(@labels, ','), ', '))" />
      </xsl:if>
    </xsl:attribute>
    <xsl:if test="@href">
      <xsl:variable name="new-href" select="if ($xsl-action='preview') then
      	concat($hosturl, psf:to-xref-page-path($siteprefix, $stylegroupname, @href, false)) else
      	@href"/>
      <xsl:attribute name="href">
        <xsl:choose>
          <xsl:when test="$ps-release != 'ps-default-value' or $ps-releasestatus != 'ps-default-value'">
            <xsl:variable name="sep" select="if (contains($new-href, '?')) then '&amp;' else '?'" />
            <xsl:variable name="before" select="if (contains($new-href, '#')) then substring-before($new-href, '#') else $new-href" />
            <xsl:variable name="after" select="if (contains($new-href, '#')) then concat('#', substring-after($new-href, '#')) else ''" />
            <xsl:variable name="params" select="if ($ps-release != 'ps-default-value' and $ps-releasestatus != 'ps-default-value') then
                                                  concat('ps-release=', $ps-release, '&amp;ps-releaseStatus=', $ps-releasestatus) else
                                                if ($ps-release != 'ps-default-value') then
                                                  concat('ps-release=', $ps-release) else concat('ps-releaseStatus=', $ps-releasestatus)" />
            <xsl:value-of select="concat($before, $sep, $params, $after)" />
          </xsl:when>
          <xsl:otherwise><xsl:value-of select="$new-href" /></xsl:otherwise>
        </xsl:choose>
      </xsl:attribute>
    </xsl:if>
		<xsl:apply-templates/>
	</xsl:element>
    <!-- the non-breaking space is so that when editing in firefox you can add text after the link easily -->
	<xsl:if test="not(starts-with(normalize-space(following::text()[1]),'&#160;'))">&#160;</xsl:if>
</xsl:template>

<xsl:template match="bold">
<b><xsl:apply-templates/></b>
</xsl:template>

<xsl:template match="sup">
<sup><xsl:apply-templates/></sup>
</xsl:template>

<xsl:template match="sub">
<sub><xsl:apply-templates/></sub>
</xsl:template>

<xsl:template match="underline">
<u><xsl:apply-templates/></u>
</xsl:template>

<xsl:template match="italic|ital">
<i><xsl:apply-templates/></i>
</xsl:template>

<xsl:template match="inlineLabel">
  <span label="{@name}" class="label-{lower-case(replace(@name, ' ', '-'))}">
    <xsl:apply-templates/>
  </span>
</xsl:template>

<xsl:template match="paraLabel">
  <div label="{@name}" class="label-{lower-case(replace(@name, ' ', '-'))}">
    <xsl:apply-templates/>
  </div>
</xsl:template>

<xsl:template match="heading1">
<h1>
	<xsl:call-template name="display-title">
		<xsl:with-param name="elem" select="." />
	</xsl:call-template>
	<xsl:apply-templates/></h1>
</xsl:template>

<xsl:template match="heading2">
<h2>
	<xsl:call-template name="display-title">
		<xsl:with-param name="elem" select="." />
	</xsl:call-template>
	<xsl:apply-templates/></h2>
</xsl:template>

<xsl:template match="heading3">
<h3>
	<xsl:call-template name="display-title">
		<xsl:with-param name="elem" select="." />
	</xsl:call-template>
	<xsl:apply-templates/></h3>
</xsl:template>

<xsl:template match="heading4">
<h4>
	<xsl:call-template name="display-title">
		<xsl:with-param name="elem" select="." />
	</xsl:call-template>
	<xsl:apply-templates/></h4>
</xsl:template>

<xsl:template match="heading5">
<h5>
	<xsl:call-template name="display-title">
		<xsl:with-param name="elem" select="." />
	</xsl:call-template>
	<xsl:apply-templates/></h5>
</xsl:template>

<xsl:template match="heading6">
<h6>
	<xsl:call-template name="display-title">
		<xsl:with-param name="elem" select="." />
	</xsl:call-template>
	<xsl:apply-templates/></h6>
</xsl:template>

<xsl:template name="display-title">
	<xsl:param name="elem"/>
	<a name="{@id}"/>
	<xsl:if test="@numbered = 'true'">
	  <xsl:value-of select="@label"/>&#160;<xsl:text />
	</xsl:if>
</xsl:template>

<xsl:template match="para">
  <p>
    <xsl:if test="@indent">
      <xsl:attribute name="class">indent<xsl:value-of select="@indent" /><xsl:if test="@label"> numbered</xsl:if></xsl:attribute>
    </xsl:if>
    <xsl:if test="@label"><span class="label"><xsl:value-of select="@label" /></span></xsl:if>
    <xsl:apply-templates/>
  </p>
</xsl:template>

<xsl:template match="monospace">
<code><xsl:apply-templates/></code>
</xsl:template>

<xsl:template match="code">
<pre><xsl:apply-templates/></pre>
</xsl:template>

<xsl:template match="list">
	<ul>
    <xsl:if test="@type"><xsl:attribute name="class"><xsl:value-of select="@type" /></xsl:attribute></xsl:if>
		<xsl:apply-templates />
	</ul>
</xsl:template>

<xsl:template match="nlist">
	<ol>
    <xsl:if test="@type"><xsl:attribute name="class"><xsl:value-of select="@type" /></xsl:attribute></xsl:if>
		<xsl:apply-templates />
	</ol>
</xsl:template>

<xsl:template match="mlist">
	<dl class="mlist">
		<xsl:apply-templates />
	</dl>
</xsl:template>

<!-- nitem include for backward compatibility only -->
<xsl:template match="item | nitem">
	<li>
		<xsl:apply-templates select="node()[not(self::list or self::nlist or self::mlist)]"/>
	</li>
	<!-- lists must be outside items for editing in Mozilla -->
	<xsl:apply-templates select="list | nlist | mlist"/>
</xsl:template>

<!-- Not required for firefox 3
<xsl:template match="mitem">
	<dt><xsl:value-of select="@label"/></dt>
	<xsl:variable name="non-list"  select="node()[not(self::list or self::nlist or self::mlist)]" />
	<xsl:if test="normalize-space($non-list[1])!=''">
		<dd>
			<xsl:apply-templates select="$non-list"/>
		</dd>
	</xsl:if>
	<!- - lists must be outside items for editing in Mozilla - ->
	<xsl:variable name="list"  select="list | nlist | mlist" />
	<xsl:if test="$list">
		<dd>
			<xsl:apply-templates select="list | nlist | mlist"/>
		</dd>
	</xsl:if>
</xsl:template>
 -->

<!-- required for correct display in firefox -->
<xsl:template match="mitem">
  <dt><xsl:value-of select="@label"/></dt>
  <dd>
    <xsl:apply-templates />
  </dd>
</xsl:template>

<xsl:template match="table">
	<table class="table">
		<xsl:copy-of select="@width | @height | @summary | @cellspacing |
                         @cellpadding | @border | @align | @summary"/>
		<xsl:variable name="colwidths" select="tgroup/colspec/@colwidth"/>
		<xsl:if test="count($colwidths) > 0">
			<xsl:attribute name="title" select="$colwidths"/>
		</xsl:if>
    <!-- Handle captions if it exists -->
    <xsl:if test="caption">
      <caption><xsl:apply-templates select="caption/node()"/></caption>
    </xsl:if>
		<tbody>
			<xsl:apply-templates select="*[not(caption)]"/>
		</tbody>
	</table>
</xsl:template>

<!-- 
  Don't do anything for captions matched (should be processed inside table). 
  Just in case it gets matched somewhere else 
 -->
<xsl:template match="caption"/>

<xsl:template match="row">
	<tr>
		<!-- xsl:if test="@role"><xsl:attribute name="style" select="@role"/></xsl:if-->
		<xsl:apply-templates/>
	</tr>
</xsl:template>

<xsl:template match="cell">
	<td class="cell">
		<xsl:copy-of select="@colspan | @rowspan | @align | @valign | @width" />
		<!--xsl:if test="@role"></xsl:if-->
		<xsl:apply-templates/>
		<xsl:if test="normalize-space(.)=''">&#160;</xsl:if>
	</td>
</xsl:template>

<xsl:template match="hcell">
	<th class="hcell" id="{@id}">
		<xsl:copy-of select="@colspan | @rowspan | @align | @valign | @width" />
		<!--xsl:if test="@role"></xsl:if-->
		<xsl:apply-templates/>
		<xsl:if test="normalize-space(.)=''">&#160;</xsl:if>
	</th>
</xsl:template>

<xsl:template match="dfx:ins">
	<ins>
		<xsl:apply-templates/>
	</ins>
</xsl:template>

<xsl:template match="dfx:del">
	<del>
		<xsl:apply-templates/>
	</del>
</xsl:template>
  
<xsl:template match="*[@dfx:insert = 'true' and node()[1]/@dfx:delete = 'true']" priority="1000000000">
  <xsl:variable name="result"><xsl:next-match/></xsl:variable>
  <xsl:for-each select="$result/*">
    <xsl:copy>
      <xsl:attribute name="class"><xsl:value-of select="@class" /> modified</xsl:attribute>
      <xsl:copy-of select="@*[name() != 'class']" />
      <xsl:copy-of select="node()" />
    </xsl:copy>
  </xsl:for-each>
</xsl:template>

<xsl:template match="*[@dfx:delete = 'true' and empty(preceding-sibling::*) and ../@dfx:insert = 'true']">
  <xsl:apply-templates />
</xsl:template>

<xsl:template match="graphic">

  <img>
    <xsl:choose>
      <xsl:when test="starts-with(@src,'/') or contains(@src,'://')">
        <xsl:attribute name="src" select="@src"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:variable name="uriFolder" select="substring-before($uriString,$filename)"/>
        <xsl:attribute name="src" select="concat(uriFolder,@src)"/>
      </xsl:otherwise>
    </xsl:choose>
      <xsl:copy-of select="@width | @height | @alt"/>
  </img>
</xsl:template>

<xsl:template match="br">
	<br/>
</xsl:template>

</xsl:stylesheet>

