/**
 * Behaviour of the project files page
 * 
 * @author Christophe Lauret
 * @version 25 October 2010
 */

// Declare all global dependencies for JSLint
/*global PAGESEEDER, $ */

// Defines and invokes an anonymous function to create a closure.
$(document).ready(function() {

  // Our container to control the tree
  var $treecontrol = $('<div class="treecontrol"></div>');
  $('#project-files').before($treecontrol);

  // Expand / Collapse Buttons
  var $expand = $('<a class="expand-all" href="#">Expand All</a>').click(function() {
    $('#ps-content-project .ps-tree-node .collapsed').toggleClass('collapsed', false).toggleClass('expanded', true);
    $('#ps-content-project .ps-tree-node-content').toggle(true);
    return false;
  }).click();
  var $collapse = $('<a class="collapse-all" href="#">Collapse All</div>').click(function() {
    $('#ps-content-project .ps-tree-node .expanded').toggleClass('collapsed', true).toggleClass('expanded', false);
    $('#ps-content-project .ps-tree-node-content').toggle(false);
    return false;
  });
  $treecontrol.append($expand).append($collapse);

  // Export
  var $export = $('<a class="export" href="#">Export</a>').click(function() {
    $.get(PublisherContext+'/admin/resources/export.xml', {'project': PublisherProject},
      function (data) {
        var filename = $(data).find('export').attr('file');
        var project = $(data).find('export').attr('project');
        window.location = PublisherContext+'/admin/resource?location=/WEB-INF/template/'+project+'/.export/'+filename+'&download=true';
      });
    return false;
  });
  $treecontrol.append($export);

  // File dropping supported
  if ($.fn.psDropFile.supported) {

    // Where the files are dropped
    var $zone = $('<div id="dropzone">Drop files here</div>');
    $('#project-files').before($zone);
    $zone.psDropFile({
      url: PublisherContext+'/admin/upload?&project='+PublisherProject,
      method: 'PUT',
      hoverClass: 'hover',
      uploadRateRefreshTime : 1000
    });

    // Invoked when the upload for given file has been started
    $.fn.psDropFile.uploadStarted = function(fileIndex, file) {
      var $li = $('<li class="ps-tree-node">');
      var $info = $('<div id="uploadinfo-'+fileIndex+'" class="ps-tree-node-file leaf"><a href="#" class="ps-tree-node-title">'+
          '<span class="file">'+(file.name || file.fileName)+'&nbsp;<span class="size">0 bytes</span></span></a></div>');
      $li.append($info);
      var $bar = $('<div id="progressbar-'+fileIndex+'"></div>');
      $info.append($bar);
      $bar.progressbar({value: 0});
      $('#dropbox ul').append($li);
    };

    // invoked when the upload for given file has been finished
    $.fn.psDropFile.uploadFinished = function(fileIndex, file, time, xhr) {
      // Remove the progress bar
      $('#progressbar-'+fileIndex).progressbar('value', 100).remove();
      // No error we upload the node
      if (xhr.status < 400) {
        var $f = $(xhr.responseXML).find('file');
        var media = $f.attr('media-type').replace(/\W|\_/g, '-');
        var zip = $f.attr('media-type') === 'application/zip';
        var $node = $('#uploadinfo-'+fileIndex);
        var $actions = $('<span class="actions">'+
            '<a title="Delete file" class="delete" href="#delete">X</a>'+
            '<a title="Rename file" class="rename" href="#rename">R</a>'+
            '<a title="Show file history" class="history" href="#history">H</a>'+
            (zip ? '<a title="Import" class="import" href="#import">I</a>':'')+'</span>');
        $node.parent().attr('location', $f.attr('path'));
        $node.find('.ps-node-title').attr('href', PublisherContext+'/admin/resources/get.html?&location='+$f.attr('path'));
        $node.draggable(draggableFileOptions).addClass(media);
        // update loaded size
        var size = $f.attr('length') > 1000? Math.round($f.attr('length') / 1000)+' kB' : $f.attr('length')+' bytes';
        $node.find(' .size').text(size);
        $node.append($actions).append('<span class="modified">A moment ago</span>');
        $node.find('.delete').click(triggerDelete);
        $node.find('.rename').click(triggerRename);
        $node.find('.history').click(viewFileHistory);
        $node.find('.import').click(triggerImport);
      // Something happened
      } else {
        var $delete = $('<span class="actions"><a title="Delete file" class="delete" href="#delete">X</a></span>');
        $delete.click(function() { $(this).closest('li').hide('puff'); });
        $('#uploadinfo-'+fileIndex).append($delete).append('<span class="modified">Error '+xhr.status+'</span>');
      }
    };

    // invoked when the progress for given file has changed
    $.fn.psDropFile.fileUploadProgressUpdated = function(fileIndex, file, progress) {
      if (progress.lengthComputable) {
        // update progress bar
        var percentage = Math.round((progress.loaded * 100) / progress.total);
        $('#progressbar-'+fileIndex).progressbar('value', percentage);
        // update loaded size
        var size = progress.loaded > 1000? Math.round(progress.loaded / 1000)+' kB' : progress.loaded+' bytes';
        $('#uploadinfo-'+fileIndex+' .size').text(size);
      }
    };

  }

  /**
   * Create a new file node in the tree.
   * @param $f The jQuery file XML from the service that created the file.
   */
  var newFile = function ($f) {
    var file = {
      location : $f.attr('path'),
      name : $f.attr('name'),
      type : $f.attr('type'),
      size : 0,
      media : $f.attr('media-type'),
      modified : 'A moment ago'
    };
    var media = file.media? file.media.replace(/\W|\_/g, '-') : 'empty-folder';
    var $li = $('<li class="ps-tree-node" location="'+file.location+'">');
    var actions, $node;
    // A new folder
    if (file.type === 'folder') {
      actions = '<a title="Delete file" class="delete" href="#delete">X</a>'+
      '<a title="Upload file" class="upload" href="#upload">U</a>'+
      '<a title="Create new file" class="create" href="#create">C</a>';
      $node = $('<div class="ps-tree-node-folder expanded ps-droppable empty-folder">'+
          '<a class="button" title="Expand / Collapse" href="#"/>'+
          '<span class="ps-tree-node-title">'+file.name+'</span>'+
          '<span class="actions">'+actions+'</span></div>');
      $node.find('.upload').click(triggerUpload);
      $node.find('.create').click(triggerCreate);
      $node.droppable(droppableFileOptions);
    // A new file
    } else if (file.type === 'file') {
      actions = '<a title="Delete file" class="delete" href="#delete">X</a>'+
                '<a title="Rename file" class="rename" href="#rename">R</a>'+
                '<a title="Show file history" class="history" href="#history">H</a>';
      if (file.media == 'text/xml') {
        actions += '<a href="'+PublisherContext+'/admin/query.html?path='+escape(file.location)+'&type=ant" '+
                   'class="run" title="Run file \''+file.name+'\' as an ANT script"><span>P</span></a>';
      }
      
      $node = $('<div class="ps-tree-node-'+file.type+' leaf '+media+'">'+
          '<a href="'+PublisherContext+'/admin/resources/get.html?&location='+file.location+'" class="ps-tree-node-title">'+
          '<span class="file">'+file.name+'&nbsp;<span class="size">'+file.size+' bytes</span></span></a>'+
          '<span class="actions">'+actions+'</span>'+
          '<span class="modified">'+file.modified+'</span></div>');
      $node.find('.rename').click(triggerRename);
      $node.find('.history').click(viewFileHistory);
      $node.draggable(draggableFileOptions);
    }
    $node.find('.delete').click(triggerDelete);
    $li.append($node);
    if (file.type === 'folder') { $li.append('<div class="ps-tree-node-content"><ul></ul></div>'); }
    return $li;
  };

  // Import =======================================================================================

  /**
   * Sends a request to import the file at the specified location.
   */
  var importPack = function (pack) {
    $.ajax({
      type: "POST",
      url: PublisherContext+'/admin/resources/import.xml',
      data: {'pack' : pack},
      success: function(data) {
        window.location.reload();
      },
      error: function(xhr, status, error){
        alert($(xhr.responseXML).find('message').text());
      }
    });
  };

  /**
   * Prompts the user to ask whether to unpack the content of the kit pack.
   */
  var triggerImport = function() {
    var $button = $(this);
    var $node = $button.closest('.ps-tree-node');
    var $confirm = $('<div>Are you sure you would like to unpack and import this file?</div>');
    $confirm.dialog({
      title: 'Unpack file',
      buttons: {
        "Unpack": function() {
          importPack($node.attr('location')); 
          $(this).dialog("close");
        },
        "Cancel": function() {
          $(this).dialog("close"); 
        }
      }
    });
    return false;
  };

  // Bind import action
  $('.actions .import').click(triggerImport);

  // Upload =======================================================================================

  /**
   * Uploads the file by submitting the form.
   */
  var uploadFile = function (form, $dialog) {
    var $form = $(form);
    var targetId = 'upload-xyz';
	  var $iframe = $('<iframe id="'+targetId+'" name="'+targetId+'" style="display: none;"></iframe>');
	  $form.attr('target', targetId);
	  $form.after($iframe);
	  $iframe.load(function() {
	    $form.find('input[type=file]').hide();
	    $form.append('<p>File was successfully uploaded!</p>');
	    $dialog.dialog('option', 'buttons', {
	      "Ok & Refresh": function() { $(this).dialog("close"); window.location.reload(); }
	    }); 
	  });
	  form.submit();
  };

  /**
   * Displays upload form.
   */
  var triggerUpload = function() {
    var $button = $(this);
    var $node = $button.closest('.ps-tree-node');
    var location = $node.attr('location');
    var uploadServlet = PublisherContext+'/admin/upload';
    var $form = $('<form id="resource-upload-form" action="'+uploadServlet+'" enctype="multipart/form-data" method="POST"></form>');
    $form.append('<p>Select the file to upload into <b>'+location.substring(location.lastIndexOf('/')+1)+'</b></p>');
    $form.append('<input name="location" type="hidden" readonly="readonly" value="'+location+'"/></input>');
    $form.append('<input name="file-1" type="file" /></input>');
    $form.dialog({
      title: 'Upload file',
      modal: true,
      buttons: {
        "Upload": function() {
          uploadFile($form.get(0), $(this));
          $form.get(0).submit();
        },
        "Cancel": function() {
          $(this).dialog("close"); 
        }
      }
    });
    return false;
  };

  // Actions on folder
  $('.actions .upload').click(triggerUpload);

  // Create =======================================================================================
  
  /**
   * Sends a request to create the file at the specified location.
   */
  var createFile = function (node, name) {
	  $.ajax({
      type: "POST",
	    url: PublisherContext+'/admin/resources/create.xml',
	    data: {'location' : $(node).attr('location')+'/'+name},
      complete: function(xhr, textStatus) {
        if (xhr.status >= 400) {
          alert($(xhr.responseXML).find('message').text());
        } else {
          var $f = $(xhr.responseXML).find('file');
          $(node).children('.ps-tree-node-content').children('ul').append(newFile($f));
        }
      }
    });
  };

  /**
   * Triggers the creation of a new resource (opens dialog for details)
   */
  var triggerCreate = function () {
    var $button = $(this);
    var $node = $button.closest('.ps-tree-node');
    var $form = $('<form id="create-resource-dialog" onsubmit="return false;"></form>');
    $form.append('<p>Enter the name of the file/folder you would like to create:</p>');
    $form.append('<p><input name="name" type="text"></input></p>');
    $form.dialog({
      title: 'Create file',
      buttons: { 
        "Create": function() {
          var name = $form.find('input').val();
          if (name.match(/\w[\w\.]+/m)) {
            createFile($node, name);
            $form.dialog("close");
          } else {
            $form.append('<p>Invalid file name!</p>');
          }
        },
        "Cancel": function() {
          $(this).dialog("close"); 
        }
      }
    });
    $form.find('input').focus().keyup(function(event) {
      if (event.keyCode == '13') {
        event.preventDefault();
        $form.dialog("option", "buttons").Create();
      }
    });
    return false;
  };

  // Actions folder
  $('.actions .create').click(triggerCreate);

  // Delete ======================================================================================

  /**
   * Sends a request to delete the file at the specified location.
   */
  var deleteFile = function (location, $node) {
	  $.ajax({
      type: "POST",
	    url: PublisherContext+'/admin/resources/delete.xml',
	    data: {'location' : location},
      success: function(data) { $node.hide('puff'); },
      error: function(xhr, status, error) {
        var msg = $(xhr.responseXML).find('message').text();
        alert(!msg ? xhr.statusText : msg);
      }
    });
  };

  /**
   * Triggers the deletion of a resource (opens dialog for confirmation)
   */
  var triggerDelete = function() {
    var $button = $(this);
    var $node = $button.closest('.ps-tree-node');
    var $dialog = $('<div id="delete-resource-dialog">');
    $dialog.append('<p>Are you sure you want to delete this file?</p>');
    $dialog.append('<p><b>'+$node.attr('location')+'</b></p>');
    $dialog.dialog({
      title: 'Delete file?',
      buttons: {
        "YES, delete it!": function() {
          deleteFile($node.attr('location'), $node);
          $(this).dialog("close"); 
        },
        "NO!": function() {
          $(this).dialog("close"); 
        }
      }
    });
    return false;
  };

  // Actions on files and folder
  $('.actions .delete').click(triggerDelete);

  // Rename =======================================================================================

  /**
   * Sends a request to delete the file at the specified location.
   * 
   * @param from  the location of the resource to rename
   * @param to    the new location of the resource
   * @param $node the 'li.ps-tree-node' being renamed 
   */
  var renameFile = function (from, to, $node) {
    $.ajax({
      type: "POST",
      url: PublisherContext+'/admin/resources/move.xml',
      data: {'location' : from, 'to': to },
      success: function(data) {
        var $f = $(data).find('file');
        var location = $f.attr('path');
        var name = location.substring(location.lastIndexOf('/')+1);
        var size = $node.find('span.file .size').text();
        $node.attr('location', location);
        $node.find('.ps-tree-node-title').attr('href', PublisherContext+'/admin/resources/get.html?location='+location);
        $node.find('span.file').text(name+' ').append('<span class="size">'+size+'</span>');
      },
      error: function(xhr, status, error){
        alert($(xhr.responseXML).find('message').text());
      }
    });
  };

  // Actions on files and folder
  var triggerRename = function () {
    var $button = $(this);
    var $node = $button.closest('.ps-tree-node');
    var location = $node.attr('location');
    var folder = location.substring(0, location.lastIndexOf('/')+1);
    var name = location.substring(location.lastIndexOf('/')+1);
    var $form = $('<form id="resource-rename-form" action="#" onsubmit="return false;"></form>');
    $form.append('<p>Where would you like to rename <b>'+name+'</b>?</p>');
    $form.append('<input name="to" type="text" value="'+name+'"/></input>');
    $form.dialog({
      title: 'Rename file',
      buttons: {
        "Rename": function() {
          var newname = $form.find('input').val();
          if (newname.match(/\w[\w\.]+/m)) {
            renameFile($node.attr('location'), folder+newname, $node);
            $form.dialog("close");
          } else {
            $form.append('<p>Invalid file name!</p>');
          }
        },
        "Cancel": function() {
          $(this).dialog("close"); 
        }
      }
    });
    $form.find('input').focus().keyup(function(event) {
      if (event.keyCode == '13') {
        event.preventDefault();
        $form.dialog("option", "buttons").Rename();
      }
    });
    return false;
  };

  $('.actions .rename').click(triggerRename);

  // Move =========================================================================================

  /**
   * Sends a request to delete the file at the specified location.
   * 
   * @param from   the location of the file to move
   * @param to     the location of the destincation folder
   * @param file   the '.ps-tree-node-file' jquery object
   * @param folder the '.ps-tree-node-folder' jquery object
   */
  var moveFile = function (from, to, file, folder) {
	  $.ajax({
      type: "POST",
	    url: PublisherContext+'/admin/resources/move.xml',
	    data: {'location' : from, 'to': to},
      success: function(data) {
        var xml = $(data);
        if (file && folder) {
          $(file).parent().attr('location', xml.find('file').attr('path'));
          $(folder).next().children('ul').append($(file).parent());
          $(folder).removeClass('empty-folder');
    	  }
      },
      error: function(xhr, status, error){
        alert($(xhr.responseXML).find('message').text());
      }
    });
  };

  var triggerMove = function() {
    var $button = $(this);
    var $node = $button.closest('.ps-tree-node');
    var $form = $('<form id="resource-move-form" action="'+$button.attr('href')+'" method="POST"></form>');
    $form.append('<p>Where would you like to move <b>'+$node.attr('location')+'</b>?</p>');
    $form.append('<input name="to" type="text" value="'+$node.attr('location')+'"/></input>');
    $form.dialog({
      title: 'Move file',
      buttons: { 
        "Move It!": function() {
          moveFile($node.attr('location'), $form.find('input').val());
          $(this).dialog("close");
        },
        "Cancel": function() {
          $(this).dialog("close"); 
        }
      }
    });
    return false;
  };

  // Actions on files and folder
  $('.actions .move').click(triggerMove);
  
  // Draggable files
  var draggableFileOptions = {
    cancel: ".actions",
    revert: "invalid",
    containment: "#project-files",
    helper: "clone",
    cursor: "move"
  };
  $(".ps-tree-node-file").draggable(draggableFileOptions);

  // Droppable files (only files for now)
  var droppableFileOptions = {
    accept: ".ps-tree-node-file",
    activeClass: "filedrop",
    hoverClass: "drophover",
    drop: function (event, ui) {
      var from = $(ui.draggable).parent().attr('location');
      var to = $(this).parent().attr('location');
      moveFile(from, to, $(ui.draggable), $(this));
    }
  };
  $(".ps-droppable").droppable(droppableFileOptions);

  // History ======================================================================================

  /**
   * Sends a request to delete the file at the specified location.
   */
  var viewFileHistory = function () {
    var $button = $(this);
    var $node = $button.closest('.ps-tree-node');
    var location = $node.attr('location');
    $.ajax({
      type: "GET",
      url: PublisherContext+'/admin/resources/history.xml',
      data: {'location' : location},
      success: function(data) {
        var $history = $('<div id="file-history"></div>');
        $history.append('<p>Change history of <b>'+location.substring(location.lastIndexOf('/')+1)+'</b>:</p>');
        var $table = $('<table>');
        $table.append('<tr><th>Revision</th><th>Size</th><th>Last Modified</th></tr>');
        $history.append($table);
        $(data).find('file').each(function() {
          var name = $(this).attr('name');
          var date = new Date();
          date.setTime(Date.parse($(this).attr('modified')));
          $table.append('<tr><th>'+name.substring(name.lastIndexOf('.r')+2)+'</th><td>'+$(this).attr('length')+' bytes</td><td>'+date.toLocaleString()+'</td></tr>');
        });      
        $history.dialog({
          title: 'File History',
          buttons: {"OK": function() { $(this).dialog("close");}}
        });
      },
      error: function(xhr, status, error){
        alert($(xhr.responseXML).find('message').text());
      }
    });
    return false;
  };

  // Actions on files and folder
  $('.actions .history').click(viewFileHistory);
  
});
